(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var MongoInternals = Package.mongo.MongoInternals;
var Mongo = Package.mongo.Mongo;
var Random = Package.random.Random;
var check = Package.check.check;
var Match = Package.check.Match;
var _ = Package.underscore._;
var Accounts = Package['accounts-base'].Accounts;

/* Package-scope variables */
var SandstormDb, Packages, DevApps, UserActions, Grains, RoleAssignments, Contacts, Sessions, SignupKeys, ActivityStats, DeleteStats, FileTokens, ApiTokens, Notifications, StatsTokens, Misc, Settings, Migrations, isDemoUser, isSignedUp, isSignedUpOrDemo, isUserOverQuota, isUserExcessivelyOverQuota, isAdmin, isAdminById, findAdminUserForToken, matchWildcardHost, makeWildcardHost, getWildcardOrigin, allowDevAccounts, roleAssignmentPattern;

(function () {

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// packages/sandstorm-db/db.js                                                                        //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                      //
// Sandstorm - Personal Cloud Sandbox                                                                 // 1
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors                              // 2
// All rights reserved.                                                                               // 3
//                                                                                                    // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                    // 5
// you may not use this file except in compliance with the License.                                   // 6
// You may obtain a copy of the License at                                                            // 7
//                                                                                                    // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                       // 9
//                                                                                                    // 10
// Unless required by applicable law or agreed to in writing, software                                // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                  // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                           // 13
// See the License for the specific language governing permissions and                                // 14
// limitations under the License.                                                                     // 15
                                                                                                      // 16
// This file defines the database schema.                                                             // 17
                                                                                                      // 18
// Users = new Mongo.Collection("users");                                                             // 19
// The users collection is special and can be accessed through `Meteor.users`.                        // 20
// See https://docs.meteor.com/#/full/meteor_users. Entries in the users collection correspond to     // 21
// accounts. Each represents a capability store belonging to a human.                                 // 22
//                                                                                                    // 23
// Note that accounts are not the same thing as identities. An account may have multiple identities   // 24
// attached to it. Identities provide a way to authenticate the user and are also used to present     // 25
// globally unique and stable user indentifiers to grains via `Grain.UserInfo.userId`.                // 26
//                                                                                                    // 27
// Each entry in this collection contains:                                                            // 28
//   _id: Random.                                                                                     // 29
//   createdAt: Date when this entry was added to the collection.                                     // 30
//   lastActive: Date of the user's most recent interaction with this Sandstorm server.               // 31
//   profile: Object containing profile data editable by the user and visible to other users. May     // 32
//            include the following fields:                                                           // 33
//       name: String containing the display name of the user.                                        // 34
//   services: Object containing login and identity data used by Meteor authentication services.      // 35
//   isAdmin: Boolean indicating whether this user is allowed to access the Sandstorm admin panel.    // 36
//   signupKey: If this is an invited user, then this field contains their signup key.                // 37
//   signupNote: If the user was invited through a link, then this field contains the note that the   // 38
//               inviter admin attached to the key.                                                   // 39
//   signupEmail: If the user was invited by email, then this field contains the email address that   // 40
//                the invite was sent to.                                                             // 41
//   quota: Number of bytes this user is allowed to store.                                            // 42
//   storageUsage: Number of bytes this user is currently storing.                                    // 43
//   expires: Date when this user's account should be deleted. Only present for demo users.           // 44
//   isAppDemoUser: True if this is a demo user who arrived via an /appdemo/ link.                    // 45
                                                                                                      // 46
Packages = new Mongo.Collection("packages");                                                          // 47
// Packages which are installed or downloading.                                                       // 48
//                                                                                                    // 49
// Each contains:                                                                                     // 50
//   _id:  128-bit prefix of SHA-256 hash of spk file, hex-encoded.                                   // 51
//   status:  String.  One of "download", "verify", "unpack", "analyze", "ready", "failed", "delete"  // 52
//   progress:  Float.  -1 = N/A, 0-1 = fractional progress (e.g. download percentage),               // 53
//       >1 = download byte count.                                                                    // 54
//   error:  If status is "failed", error message string.                                             // 55
//   manifest:  If status is "ready", the package manifest.  See "Manifest" in grain.capnp.           // 56
//   appId:  If status is "ready", the application ID string.  Packages representing different        // 57
//       versions of the same app have the same appId.  The spk tool defines the app ID format        // 58
//       and can cryptographically verify that a package belongs to a particular app ID.              // 59
                                                                                                      // 60
DevApps = new Mongo.Collection("devapps");                                                            // 61
// List of applications currently made available via the dev tools running on the local machine.      // 62
// This is normally empty; the only time it is non-empty is when a developer is using the spk tool    // 63
// on the local machine to publish an under-development app to this server. That should only ever     // 64
// happen on developers' desktop machines.                                                            // 65
//                                                                                                    // 66
// While a dev app is published, it automatically appears as installed by every user of the server,   // 67
// and it overrides all packages with the same application ID. If any instances of those packages     // 68
// are currently open, they are killed and reset on publish.                                          // 69
//                                                                                                    // 70
// When the dev tool disconnects, the app is automatically unpublished, and any open instances        // 71
// are again killed and refreshed.                                                                    // 72
//                                                                                                    // 73
// Each contains:                                                                                     // 74
//   _id:  The application ID string (as with Packages.appId).                                        // 75
//   packageId:  The directory name where the dev package is mounted.                                 // 76
//   timestamp:  Time when the package was last updated. If this changes while the package is         // 77
//     published, all running instances are reset. This is used e.g. to reset the app each time       // 78
//     changes are made to the source code.                                                           // 79
//   manifest:  The app's manifest, as with Packages.manifest.                                        // 80
                                                                                                      // 81
UserActions = new Mongo.Collection("userActions");                                                    // 82
// List of actions that each user has installed which create new grains.  Each app may install        // 83
// some number of actions (usually, one).                                                             // 84
//                                                                                                    // 85
// Each contains:                                                                                     // 86
//   _id:  random                                                                                     // 87
//   userId:  User who has installed this action.                                                     // 88
//   packageId:  Package used to run this action.                                                     // 89
//   appId:  Same as Packages.findOne(packageId).appId; denormalized for searchability.               // 90
//   appTitle:  Same as Packages.findOne(packageId).manifest.appTitle.defaultText; denormalized so    // 91
//       that clients can access it without subscribing to the Packages collection.                   // 92
//   appVersion:  Same as Packages.findOne(packageId).manifest.appVersion; denormalized for           // 93
//       searchability.                                                                               // 94
//   appMarketingVersion:  Human-readable presentation of the app version, e.g. "2.9.17"              // 95
//   title:  Human-readable title for this action, e.g. "New Spreadsheet".                            // 96
//   command:  Manifest.Command to run this action (see package.capnp).                               // 97
                                                                                                      // 98
Grains = new Mongo.Collection("grains");                                                              // 99
// Grains belonging to users.                                                                         // 100
//                                                                                                    // 101
// Each contains:                                                                                     // 102
//   _id:  random                                                                                     // 103
//   packageId:  _id of the package of which this grain is an instance.                               // 104
//   appId:  Same as Packages.findOne(packageId).appId; denormalized for searchability.               // 105
//   appVersion:  Same as Packages.findOne(packageId).manifest.appVersion; denormalized for           // 106
//       searchability.                                                                               // 107
//   userId:  User who owns this grain.                                                               // 108
//   title:  Human-readable string title, as chosen by the user.                                      // 109
//   lastUsed:  Date when the grain was last used by a user.                                          // 110
//   private: If true, then knowledge of `_id` does not suffice to open this grain.                   // 111
//   cachedViewInfo: The JSON-encoded result of `UiView.getViewInfo()`, cached from the most recent   // 112
//                   time a session to this grain was opened.                                         // 113
//                                                                                                    // 114
// The following fields *might* also exist. These are temporary hacks used to implement e-mail and    // 115
// web publishing functionality without powerbox support; they will be replaced once the powerbox     // 116
// is implemented.                                                                                    // 117
//   publicId:  An id used to publicly identify this grain. Used e.g. to route incoming e-mail and    // 118
//       web publishing. This field is initialized when first requested by the app.                   // 119
                                                                                                      // 120
RoleAssignments = new Mongo.Collection("roleAssignments");                                            // 121
// *OBSOLETE* Before `user` was a variant of ApiTokenOwner, this collection was used to store edges   // 122
// in the permissions sharing graph. This functionality has been subsumed by the ApiTokens            // 123
// collection.                                                                                        // 124
                                                                                                      // 125
Contacts = new Mongo.Collection("contacts");                                                          // 126
// Edges in the social graph.                                                                         // 127
//                                                                                                    // 128
// If Alice has Bob as a contact, then she is allowed to see Bob's profile information and Bob        // 129
// will show up in her user-picker UI for actions like share-by-identity.                             // 130
//                                                                                                    // 131
// Contacts are not symmetric. Bob might be one of Alice's contacts even if Alice is not one of       // 132
// Bob's.                                                                                             // 133
//                                                                                                    // 134
// Each contains:                                                                                     // 135
//   _id: random                                                                                      // 136
//   ownerId: The `_id` of the user who owns this contact.                                            // 137
//   userId:  The `_id` of the contacted user.                                                        // 138
//   petname: Human-readable label chosen by and only visible to the owner. Uniquely identifies       // 139
//            the contact to the owner.                                                               // 140
//   created: Date when this contact was created.                                                     // 141
                                                                                                      // 142
Sessions = new Mongo.Collection("sessions");                                                          // 143
// UI sessions open to particular grains.  A new session is created each time a user opens a grain.   // 144
//                                                                                                    // 145
// Each contains:                                                                                     // 146
//   _id:  random                                                                                     // 147
//   grainId:  _id of the grain to which this session is connected.                                   // 148
//   hostId: ID part of the hostname from which this grain is being served. I.e. this replaces the    // 149
//       '*' in WILDCARD_HOST.                                                                        // 150
//   timestamp:  Time of last keep-alive message to this session.  Sessions time out after some       // 151
//       period.                                                                                      // 152
//   userId:  User who owns this session.                                                             // 153
//   hashedToken: If the session is owned by an anonymous user, the _id of the entry in ApiTokens     // 154
//       that was used to open it. Note that for old-style sharing (i.e. when !grain.private),        // 155
//       anonymous users can get access without an API token and so neither userId nor hashedToken    // 156
//       are present.                                                                                 // 157
//   powerboxView: If present, this is a view that should be presented as part of a powerbox          // 158
//       interaction.                                                                                 // 159
//     offer: The webkey that corresponds to cap that was passed to the `offer` RPC.                  // 160
                                                                                                      // 161
SignupKeys = new Mongo.Collection("signupKeys");                                                      // 162
// Invite keys which may be used by users to get access to Sandstorm.                                 // 163
//                                                                                                    // 164
// Each contains:                                                                                     // 165
//   _id:  random                                                                                     // 166
//   used:  Boolean indicating whether this key has already been consumed.                            // 167
//   note:  Text note assigned when creating key, to keep track of e.g. whom the key was for.         // 168
//   email: If this key was sent as an email invite, the email address to which it was sent.          // 169
//   quota: If present, the storage quota to assign to the user claiming this invite.                 // 170
                                                                                                      // 171
ActivityStats = new Mongo.Collection("activityStats");                                                // 172
// Contains usage statistics taken on a regular interval. Each entry is a data point.                 // 173
//                                                                                                    // 174
// Each contains:                                                                                     // 175
//   timestamp: Date when measurements were taken.                                                    // 176
//   daily: Contains stats counts pertaining to the last day before the sample time.                  // 177
//   weekly: Contains stats counts pertaining to the last seven days before the sample time.          // 178
//   monthly: Contains stats counts pertaining to the last thirty days before the timestamp.          // 179
//                                                                                                    // 180
// Each of daily, weekly, and monthly contains:                                                       // 181
//   activeUsers: The number of unique users who have used a grain on the server in the time          // 182
//       interval. Only counts logged-in users.                                                       // 183
//   activeGrains: The number of unique grains that have been used in the time interval.              // 184
                                                                                                      // 185
DeleteStats = new Mongo.Collection("deleteStats");                                                    // 186
// Contains records of objects that were deleted, for stat-keeping purposes.                          // 187
//                                                                                                    // 188
// Each contains:                                                                                     // 189
//   type: "grain" or "user" or "appDemoUser"                                                         // 190
//   lastActive: Date of the user's or grain's last activity.                                         // 191
                                                                                                      // 192
FileTokens = new Mongo.Collection("fileTokens");                                                      // 193
// Tokens corresponding to backup files that are currently stored on the server. A user receives      // 194
// a token when they create a backup file (either by uploading it, or by backing up one of their      // 195
// grains) and may use the token to read the file (either to download it, or to restore a new         // 196
// grain from it).                                                                                    // 197
//                                                                                                    // 198
// Each contains:                                                                                     // 199
//   _id:       The unguessable token string.                                                         // 200
//   name:      Suggested filename.                                                                   // 201
//   timestamp: File creation time. Used to figure out when the token and file should be wiped.       // 202
                                                                                                      // 203
ApiTokens = new Mongo.Collection("apiTokens");                                                        // 204
// Access tokens for APIs exported by apps.                                                           // 205
//                                                                                                    // 206
// Originally API tokens were only used by external users through the HTTP API endpoint. However,     // 207
// now they are also used to implement SturdyRefs, not just held by external users, but also when     // 208
// an app holds a SturdyRef to another app within the same server. See the various `save()`,          // 209
// `restore()`, and `drop()` methods in `grain.capnp` (on `SansdtormApi`, `AppPersistent`, and        // 210
// `MainView`) -- the fields of type `Data` are API tokens.                                           // 211
//                                                                                                    // 212
// Each contains:                                                                                     // 213
//   _id:       A SHA-256 hash of the token.                                                          // 214
//   grainId:   The grain servicing this API. (Not present if the API isn't serviced by a grain.)     // 215
//   userId:    For UiView capabilities, this is the user for whom the view is attenuated. That       // 216
//              is, the UiView's newSession() method will intersect the requested permissions with    // 217
//              this user's permissions before forwarding on to the underlying app. If `userId` is    // 218
//              not present, then no user attenuation is applied, i.e. this is a raw UiView as        // 219
//              implemented by the app. (The `roleAssignment` field, below, may still apply.)         // 220
//              For non-UiView capabilities, `userId` is never present.                               // 221
//              Note that this is NOT the user against whom the `requiredPermissions` parameter of    // 222
//              `SandstormApi.restore()` is checked; that would be `owner.grain.introducerUser`.      // 223
//   userInfo:  *DEPRECATED* For API tokens created by the app through HackSessionContext, the        // 224
//              UserInfo struct that should be passed to `newSession()` when exercising this token,   // 225
//              in decoded (JS object) format. This is a temporary hack. `userId` is never present    // 226
//              when `userInfo` is present.                                                           // 227
//   roleAssignment: If this API token represents a UiView, this field contains a JSON-encoded        // 228
//              Grain.ViewSharingLink.RoleAssignment representing the permissions it carries. These   // 229
//              permissions will be intersected with those held by `userId` when the view is opened.  // 230
//   forSharing: If true, requests sent to the HTTP API endpoint with this token will be treated as   // 231
//              anonymous rather than as directly associated with `userId`. This has no effect on     // 232
//              the permissions granted.                                                              // 233
//   objectId:  If present, this token represents an arbitrary Cap'n Proto capability exported by     // 234
//              the app or its supervisor (whereas without this it strictly represents UiView).       // 235
//              sturdyRef is the JSON-encoded SupervisorObjectId (defined in `supervisor.capnp`).     // 236
//              Note that if the SupervisorObjectId contains an AppObjectId, that field is            // 237
//              treated as type AnyPointer, and so encoded as a raw Cap'n Proto message.              // 238
//   frontendRef: If present, this token actually refers to an object implemented by the front-end,   // 239
//              not a particular grain. (`grainId` and `userId` are not set.) This is an object       // 240
//              containing exactly one of the following fields:                                       // 241
//       notificationHandle: A `Handle` for an ongoing notification, as returned by                   // 242
//                           `NotificationTarget.addOngoing`. The value is an `_id` from the          // 243
//                           `Notifications` collection.                                              // 244
//       ipNetwork: An IpNetwork capability that is implemented by the frontend. Eventually, this     // 245
//                  will be moved out of the frontend and into the backend, but we'll migrate the     // 246
//                  database when that happens. This field contains the boolean true to signify that  // 247
//                  it has been set.                                                                  // 248
//       ipInterface: Ditto IpNetwork, except it's an IpInterface.                                    // 249
//   parentToken: If present, then this token represents exactly the capability represented by        // 250
//              the ApiToken with _id = parentToken, except possibly (if it is a UiView) attenuated   // 251
//              by `roleAssignment` (if present). To facilitate permissions computations, if the      // 252
//              capability is a UiView, then `grainId` is set to the backing grain and `userId` is    // 253
//              set to the user who shared the view. None of `userInfo`, `objectId`, or               // 254
//              `frontendRef` are present when `parentToken` is present.                              // 255
//   petname:   Human-readable label for this access token, useful for identifying tokens for         // 256
//              revocation. This should be displayed when visualizing incoming capabilities to        // 257
//              the grain identified by `grainId`.                                                    // 258
//   created:   Date when this token was created.                                                     // 259
//   revoked:   If true, then this sturdyref has been revoked and can no longer be restored. It may   // 260
//              become un-revoked in the future.                                                      // 261
//   expires:   Optional expiration Date. If undefined, the token does not expire.                    // 262
//   owner:     A `ApiTokenRefOwner` (defined in `supervisor.capnp`, stored as a JSON object)         // 263
//              as passed to the `save()` call that created this token. If not present, treat         // 264
//              as `webkey` (the default for `ApiTokenOwner`).                                        // 265
//   expiresIfUnused:                                                                                 // 266
//              Optional Date after which the token, if it has not been used yet, expires.            // 267
//              This field should be cleared on a token's first use.                                  // 268
//   requirements: List of conditions which must hold for this token to be considered valid.          // 269
//              Semantically, this list specifies the powers which were *used* to originally          // 270
//              create the token. If any condition in the list becomes untrue, then the token must    // 271
//              be considered revoked, and all live refs and sturdy refs obtained transitively        // 272
//              through it must also become revoked. Each item is the JSON serialization of the       // 273
//              `MembraneRequirement` structure defined in `supervisor.capnp`.                        // 274
//                                                                                                    // 275
// It is important to note that a token's owner and provider are independent from each other. To      // 276
// illustrate, here is an approximate definition of ApiToken in pseudo Cap'n Proto schema language:   // 277
//                                                                                                    // 278
// struct ApiToken {                                                                                  // 279
//   owner :ApiTokenOwner;                                                                            // 280
//   provider :union {                                                                                // 281
//     grain :group {                                                                                 // 282
//       grainId :Text;                                                                               // 283
//       union {                                                                                      // 284
//         uiView :group {                                                                            // 285
//           userId :Text;                                                                            // 286
//           roleAssignment :RoleAssignment;                                                          // 287
//           forSharing :Bool;                                                                        // 288
//         }                                                                                          // 289
//         objectId :SupervisorObjectId;                                                              // 290
//       }                                                                                            // 291
//     }                                                                                              // 292
//     frontendRef :union {                                                                           // 293
//        notificationHandle :Text;                                                                   // 294
//        ipNetwork :Bool;                                                                            // 295
//        ipInterface :Bool;                                                                          // 296
//     }                                                                                              // 297
//     child :group {                                                                                 // 298
//       parentToken :Text;                                                                           // 299
//       union {                                                                                      // 300
//         uiView :group {                                                                            // 301
//           grainId :Text;                                                                           // 302
//           userId :Text;                                                                            // 303
//           roleAssignment :RoleAssignment = (allAccess = ());                                       // 304
//         }                                                                                          // 305
//         other :Void;                                                                               // 306
//       }                                                                                            // 307
//     }                                                                                              // 308
//   }                                                                                                // 309
//   requirements: List(Supervisor.MembraneRequirement);                                              // 310
//   ...                                                                                              // 311
// }                                                                                                  // 312
                                                                                                      // 313
Notifications = new Mongo.Collection("notifications");                                                // 314
// Notifications for a user.                                                                          // 315
//                                                                                                    // 316
// Each contains:                                                                                     // 317
//   _id:          random                                                                             // 318
//   ongoing:      If present, this is an ongoing notification, and this field contains an            // 319
//                 ApiToken referencing the `OngoingNotification` capability.                         // 320
//   grainId:      The grain originating this notification, if any.                                   // 321
//   userId:       The user receiving the notification.                                               // 322
//   text:         The JSON-ified LocalizedText to display in the notification.                       // 323
//   isUnread:     Boolean indicating if this notification is unread.                                 // 324
//   timestamp:    Date when this notification was last updated                                       // 325
                                                                                                      // 326
StatsTokens = new Mongo.Collection("statsTokens");                                                    // 327
// Access tokens for the Stats collection                                                             // 328
//                                                                                                    // 329
// These tokens are used for accessing the ActivityStats collection remotely                          // 330
// (ie. from a dashboard webapp)                                                                      // 331
//                                                                                                    // 332
// Each contains:                                                                                     // 333
//   _id:       The token. At least 128 bits entropy (Random.id(22)).                                 // 334
                                                                                                      // 335
Misc = new Mongo.Collection("misc");                                                                  // 336
// Miscellaneous configuration and other settings                                                     // 337
//                                                                                                    // 338
// This table is currently only used for persisting BASE_URL from one session to the next,            // 339
// but in general any miscellaneous settings should go in here                                        // 340
//                                                                                                    // 341
// Each contains:                                                                                     // 342
//   _id:       The name of the setting. eg. "BASE_URL"                                               // 343
//   value:     The value of the setting.                                                             // 344
                                                                                                      // 345
Settings = new Mongo.Collection("settings");                                                          // 346
// Settings for this Sandstorm instance go here. They are configured through the adminSettings        // 347
// route. This collection differs from misc in that this collection is completely user controlled.    // 348
//                                                                                                    // 349
// Each contains:                                                                                     // 350
//   _id:       The name of the setting. eg. "MAIL_URL"                                               // 351
//   value:     The value of the setting.                                                             // 352
//   potentially other fields that are unique to the setting                                          // 353
                                                                                                      // 354
Migrations = new Mongo.Collection("migrations");                                                      // 355
// This table tracks which migrations we have applied to this instance.                               // 356
// It contains a single entry:                                                                        // 357
//   _id:       "migrations_applied"                                                                  // 358
//   value:     The number of migrations this instance has successfully completed.                    // 359
                                                                                                      // 360
if (Meteor.isServer) {                                                                                // 361
  Meteor.publish("credentials", function () {                                                         // 362
    // Data needed for isSignedUp() and isAdmin() to work.                                            // 363
                                                                                                      // 364
    if (this.userId) {                                                                                // 365
      return Meteor.users.find({_id: this.userId},                                                    // 366
          {fields: {signupKey: 1, isAdmin: 1, expires: 1, quota: 1, storageUsage: 1}});               // 367
    } else {                                                                                          // 368
      return [];                                                                                      // 369
    }                                                                                                 // 370
  });                                                                                                 // 371
                                                                                                      // 372
  // The first user to sign in should be automatically upgraded to admin.                             // 373
  Accounts.onCreateUser(function (options, user) {                                                    // 374
    // Dev users are identified by having the devName field                                           // 375
    // Don't count them in our find and don't give them admin                                         // 376
    if (Meteor.users.find({devName: {$exists: 0}}).count() === 0 && !user.devName) {                  // 377
      user.isAdmin = true;                                                                            // 378
      user.signupKey = "admin";                                                                       // 379
    }                                                                                                 // 380
                                                                                                      // 381
    if (options.profile) {                                                                            // 382
      user.profile = options.profile;                                                                 // 383
    }                                                                                                 // 384
                                                                                                      // 385
    return user;                                                                                      // 386
  });                                                                                                 // 387
}                                                                                                     // 388
                                                                                                      // 389
isDemoUser = function() {                                                                             // 390
  // Returns true if this is a demo user.                                                             // 391
                                                                                                      // 392
  var user = Meteor.user();                                                                           // 393
  if (user && user.expires) {                                                                         // 394
    return true;                                                                                      // 395
  } else {                                                                                            // 396
    return false;                                                                                     // 397
  }                                                                                                   // 398
}                                                                                                     // 399
                                                                                                      // 400
isSignedUp = function() {                                                                             // 401
  // Returns true if the user has presented an invite key.                                            // 402
                                                                                                      // 403
  var user = Meteor.user();                                                                           // 404
  if (user && user.signupKey) {                                                                       // 405
    return true;                                                                                      // 406
  } else {                                                                                            // 407
    return false;                                                                                     // 408
  }                                                                                                   // 409
}                                                                                                     // 410
                                                                                                      // 411
isSignedUpOrDemo = function () {                                                                      // 412
  var user = Meteor.user();                                                                           // 413
  if (user && (user.signupKey || user.expires)) {                                                     // 414
    return true;                                                                                      // 415
  } else {                                                                                            // 416
    return false;                                                                                     // 417
  }                                                                                                   // 418
}                                                                                                     // 419
                                                                                                      // 420
isUserOverQuota = function (user) {                                                                   // 421
  return Meteor.settings.public.quotaEnabled &&                                                       // 422
         typeof user.quota === "number" &&                                                            // 423
         user.storageUsage && user.storageUsage >= user.quota;                                        // 424
}                                                                                                     // 425
                                                                                                      // 426
isUserExcessivelyOverQuota = function (user) {                                                        // 427
  return Meteor.settings.public.quotaEnabled &&                                                       // 428
         typeof user.quota === "number" &&                                                            // 429
         user.storageUsage && user.storageUsage >= user.quota * 1.2;                                  // 430
}                                                                                                     // 431
                                                                                                      // 432
isAdmin = function() {                                                                                // 433
  // Returns true if the user is the administrator.                                                   // 434
                                                                                                      // 435
  var user = Meteor.user();                                                                           // 436
  if (user && user.isAdmin) {                                                                         // 437
    return true;                                                                                      // 438
  } else {                                                                                            // 439
    return false;                                                                                     // 440
  }                                                                                                   // 441
}                                                                                                     // 442
                                                                                                      // 443
isAdminById = function(id) {                                                                          // 444
  // Returns true if the user's id is the administrator.                                              // 445
                                                                                                      // 446
  var user = Meteor.users.findOne({_id: id}, {fields: {isAdmin: 1}})                                  // 447
  if (user && user.isAdmin) {                                                                         // 448
    return true;                                                                                      // 449
  } else {                                                                                            // 450
    return false;                                                                                     // 451
  }                                                                                                   // 452
}                                                                                                     // 453
                                                                                                      // 454
findAdminUserForToken = function (token) {                                                            // 455
  if (!token.requirements) {                                                                          // 456
    return;                                                                                           // 457
  }                                                                                                   // 458
  var requirements = token.requirements.filter(function (requirement) {                               // 459
    return "userIsAdmin" in requirement;                                                              // 460
  });                                                                                                 // 461
                                                                                                      // 462
  if (requirements.length > 1) {                                                                      // 463
    return;                                                                                           // 464
  }                                                                                                   // 465
  if (requirements.length === 0) {                                                                    // 466
    return;                                                                                           // 467
  }                                                                                                   // 468
  return requirements[0].userIsAdmin;                                                                 // 469
};                                                                                                    // 470
                                                                                                      // 471
var wildcardHost = Meteor.settings.public.wildcardHost.toLowerCase().split("*");                      // 472
                                                                                                      // 473
if (wildcardHost.length != 2) {                                                                       // 474
  throw new Error("Wildcard host must contain exactly one asterisk.");                                // 475
}                                                                                                     // 476
                                                                                                      // 477
matchWildcardHost = function(host) {                                                                  // 478
  // See if the hostname is a member of our wildcard. If so, extract the ID.                          // 479
                                                                                                      // 480
  var prefix = wildcardHost[0];                                                                       // 481
  var suffix = wildcardHost[1];                                                                       // 482
  if (host.lastIndexOf(prefix, 0) >= 0 &&                                                             // 483
      host.indexOf(suffix, -suffix.length) >= 0 &&                                                    // 484
      host.length >= prefix.length + suffix.length) {                                                 // 485
    var id = host.slice(prefix.length, -suffix.length);                                               // 486
    if (id.match(/^[a-z0-9]*$/)) {                                                                    // 487
      return id;                                                                                      // 488
    }                                                                                                 // 489
  }                                                                                                   // 490
                                                                                                      // 491
  return null;                                                                                        // 492
}                                                                                                     // 493
                                                                                                      // 494
makeWildcardHost = function (id) {                                                                    // 495
  return wildcardHost[0] + id + wildcardHost[1];                                                      // 496
}                                                                                                     // 497
                                                                                                      // 498
if (Meteor.isServer) {                                                                                // 499
  var Url = Npm.require("url");                                                                       // 500
  getWildcardOrigin = function () {                                                                   // 501
    // The wildcard URL can be something like "foo-*-bar.example.com", but sometimes when we're       // 502
    // trying to specify a pattern matching hostnames (say, a Content-Security-Policy directive),     // 503
    // an astrisk is only allowed as the first character and must be followed by a period. So we need // 504
    // "*.example.com" instead -- which matches more than we actually want, but is the best we can    // 505
    // really do. We also add the protocol to the front (again, that's what CSP wants).               // 506
                                                                                                      // 507
    // TODO(cleanup): `protocol` is computed in other files, like proxy.js. Put it somewhere common.  // 508
    var protocol = Url.parse(process.env.ROOT_URL).protocol;                                          // 509
                                                                                                      // 510
    var dotPos = wildcardHost[1].indexOf(".");                                                        // 511
    if (dotPos < 0) {                                                                                 // 512
      return protocol + "//*";                                                                        // 513
    } else {                                                                                          // 514
      return protocol + "//*" + wildcardHost[1].slice(dotPos);                                        // 515
    }                                                                                                 // 516
  }                                                                                                   // 517
}                                                                                                     // 518
                                                                                                      // 519
allowDevAccounts = function () {                                                                      // 520
  var setting = Settings.findOne({_id: "devAccounts"});                                               // 521
  if (setting) {                                                                                      // 522
    return setting.value;                                                                             // 523
  } else {                                                                                            // 524
    return Meteor.settings && Meteor.settings.public &&                                               // 525
           Meteor.settings.public.allowDevAccounts;                                                   // 526
  }                                                                                                   // 527
};                                                                                                    // 528
                                                                                                      // 529
roleAssignmentPattern = {                                                                             // 530
  none : Match.Optional(null),                                                                        // 531
  allAccess: Match.Optional(null),                                                                    // 532
  roleId: Match.Optional(Match.Integer),                                                              // 533
  addPermissions: Match.Optional([Boolean]),                                                          // 534
  removePermissions: Match.Optional([Boolean]),                                                       // 535
};                                                                                                    // 536
                                                                                                      // 537
SandstormDb = function () {                                                                           // 538
  this.collections = {                                                                                // 539
    // Direct access to underlying collections. DEPRECATED.                                           // 540
    //                                                                                                // 541
    // TODO(cleanup): Over time, we will provide methods covering each supported query and remove     // 542
    //   direct access to the collections.                                                            // 543
                                                                                                      // 544
    packages: Packages,                                                                               // 545
    devApps: DevApps,                                                                                 // 546
    userActions: UserActions,                                                                         // 547
    grains: Grains,                                                                                   // 548
    contacts: Contacts,                                                                               // 549
    sessions: Sessions,                                                                               // 550
    signupKeys: SignupKeys,                                                                           // 551
    activityStats: ActivityStats,                                                                     // 552
    deleteStats: DeleteStats,                                                                         // 553
    fileTokens: FileTokens,                                                                           // 554
    apiTokens: ApiTokens,                                                                             // 555
    notifications: Notifications,                                                                     // 556
    statsTokens: StatsTokens,                                                                         // 557
    misc: Misc,                                                                                       // 558
    settings: Settings,                                                                               // 559
                                                                                                      // 560
    // Intentionally omitted:                                                                         // 561
    // - Migrations, since it's used only within this package.                                        // 562
    // - RoleAssignments, since it is deprecated and only used by the migration that eliminated it.   // 563
  };                                                                                                  // 564
};                                                                                                    // 565
                                                                                                      // 566
// TODO(cleanup): These methods should not be defined freestanding and should use collection          // 567
//   objects created in SandstormDb's constructor rather than globals.                                // 568
                                                                                                      // 569
_.extend(SandstormDb.prototype, {                                                                     // 570
  isDemoUser: isDemoUser,                                                                             // 571
  isSignedUp: isSignedUp,                                                                             // 572
  isSignedUpOrDemo: isSignedUpOrDemo,                                                                 // 573
  isUserOverQuota: isUserOverQuota,                                                                   // 574
  isUserExcessivelyOverQuota: isUserExcessivelyOverQuota,                                             // 575
  isAdmin: isAdmin,                                                                                   // 576
  isAdminById: isAdminById,                                                                           // 577
  findAdminUserForToken: findAdminUserForToken,                                                       // 578
  matchWildcardHost: matchWildcardHost,                                                               // 579
  makeWildcardHost: makeWildcardHost,                                                                 // 580
  allowDevAccounts: allowDevAccounts,                                                                 // 581
  roleAssignmentPattern: roleAssignmentPattern,                                                       // 582
});                                                                                                   // 583
                                                                                                      // 584
if (Meteor.isServer) {                                                                                // 585
  SandstormDb.prototype.getWildcardOrigin = getWildcardOrigin;                                        // 586
}                                                                                                     // 587
                                                                                                      // 588
////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);






(function () {

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// packages/sandstorm-db/migrations.js                                                                //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                      //
// Minimal tooling for doing run-at-least-once, ordered migrations.                                   // 1
//                                                                                                    // 2
// Because migrations can experience partial failure and likely have                                  // 3
// side-effects, we should be careful to make sure all migrations are                                 // 4
// idempotent and safe to accidentally run multiple times.                                            // 5
                                                                                                      // 6
var updateLoginStyleToRedirect = function() {                                                         // 7
  var configurations = Package["service-configuration"].ServiceConfiguration.configurations;          // 8
  ["google", "github"].forEach(function(serviceName) {                                                // 9
    var config = configurations.findOne({service: serviceName});                                      // 10
    if (config && config.loginStyle !== "redirect") {                                                 // 11
      configurations.update({service: serviceName}, {$set: {loginStyle: "redirect"}});                // 12
    }                                                                                                 // 13
  });                                                                                                 // 14
};                                                                                                    // 15
                                                                                                      // 16
var enableLegacyOAuthProvidersIfNotInSettings = function() {                                          // 17
  // In the before time, Google and Github login were enabled by default.                             // 18
  //                                                                                                  // 19
  // This actually didn't make much sense, required the first user to configure                       // 20
  // OAuth, and had some trust-the-first-user properties that weren't totally                         // 21
  // secure.                                                                                          // 22
  //                                                                                                  // 23
  // Now, we have admin-token, so we wish to disable all logins by default                            // 24
  // (since they need to be configured anyway) but since users may have never                         // 25
  // explicitly told Sandstorm that Google or Github login should be enabled,                         // 26
  // we can't just use the value in the Settings collection, since it might                           // 27
  // never have been set.                                                                             // 28
  //                                                                                                  // 29
  // Thus, if the service is configured but absent from Settings, we should                           // 30
  // explicitly enable it in Settings, and then the rest of the logic can just                        // 31
  // depend on what value is in Settings and default to false without breaking                        // 32
  // user installations.                                                                              // 33
  var configurations = Package["service-configuration"].ServiceConfiguration.configurations;          // 34
  ["google", "github"].forEach(function(serviceName) {                                                // 35
    var config = configurations.findOne({service: serviceName});                                      // 36
    var serviceConfig = Settings.findOne({_id: serviceName});                                         // 37
    if (config && !serviceConfig) {                                                                   // 38
      // Only explicitly enable the login service if:                                                 // 39
      // 1) the service is already configured                                                         // 40
      // 2) there is no sandstorm configuration already present (the user was                         // 41
      //    using the previous default behavior).                                                     // 42
      Settings.insert({_id: serviceName, value: true});                                               // 43
    }                                                                                                 // 44
  });                                                                                                 // 45
};                                                                                                    // 46
                                                                                                      // 47
var denormalizeInviteInfo = function() {                                                              // 48
  // When a user is invited via a signup token, the `signupKey` field of their user table entry       // 49
  // has always been populated to indicate the key they used. This points into the SignupKeys table   // 50
  // which has more information about the key, namely a freeform note entered by the admin when       // 51
  // they created the key. In the case that the email invite form was used, the note has the form     // 52
  // "E-mail invite to <address>".                                                                    // 53
  //                                                                                                  // 54
  // Later, we decided it was useful to indicate in the users table visible to the admin              // 55
  // information about the invite terms. Namely, for email invites we want to show the address        // 56
  // and for others we want to show the note. To make this efficient, fields `signupNote` and         // 57
  // `signupEmail` were added to the users table. We can backfill these values by denormalizing       // 58
  // from the SignupKeys table.                                                                       // 59
                                                                                                      // 60
  Meteor.users.find().forEach(function (user) {                                                       // 61
    if (user.signupKey && (typeof user.signupKey) === "string" && user.signupKey !== "admin") {       // 62
      var signupInfo = SignupKeys.findOne(user.signupKey);                                            // 63
      if (signupInfo && signupInfo.note) {                                                            // 64
        var newFields = { signupNote: signupInfo.note };                                              // 65
                                                                                                      // 66
        var prefix = "E-mail invite to ";                                                             // 67
        if (signupInfo.note.lastIndexOf(prefix) === 0) {                                              // 68
          newFields.signupEmail = signupInfo.note.slice(prefix.length);                               // 69
        }                                                                                             // 70
                                                                                                      // 71
        Meteor.users.update(user._id, {$set: newFields});                                             // 72
      }                                                                                               // 73
    }                                                                                                 // 74
  });                                                                                                 // 75
}                                                                                                     // 76
                                                                                                      // 77
function mergeRoleAssignmentsIntoApiTokens() {                                                        // 78
  RoleAssignments.find().forEach(function(roleAssignment) {                                           // 79
    ApiTokens.insert({                                                                                // 80
      grainId: roleAssignment.grainId,                                                                // 81
      userId: roleAssignment.sharer,                                                                  // 82
      roleAssignment: roleAssignment.roleAssignment,                                                  // 83
      petname: roleAssignment.petname,                                                                // 84
      created: roleAssignment.created,                                                                // 85
      owner: {user: {userId: roleAssignment.recipient,                                                // 86
                     title: roleAssignment.title}},                                                   // 87
    });                                                                                               // 88
  });                                                                                                 // 89
}                                                                                                     // 90
                                                                                                      // 91
function fixOasisStorageUsageStats() {                                                                // 92
  // Before storage quota enforcement was first implemented, there were some bugs with the way the    // 93
  // Oasis backend tracked storage usage, which were not initially noticed since the resulting        // 94
  // number wasn't acutally used for anything.                                                        // 95
  //                                                                                                  // 96
  // With the underlying bugs fixed, simply starting up each grain (supervisor only, not the app)     // 97
  // and then immediately shutting it down should cause the sizes to be recalculated correctly.       // 98
  //                                                                                                  // 99
  // This problem only applies to Oasis.                                                              // 100
                                                                                                      // 101
  if (Meteor.settings.public.quotaEnabled) {                                                          // 102
    var dummyCommand = {                                                                              // 103
      argv: ["foo", "bar"],                                                                           // 104
      environ: []                                                                                     // 105
    };                                                                                                // 106
                                                                                                      // 107
    Grains.find().forEach(function (grain) {                                                          // 108
      try {                                                                                           // 109
        console.log("refreshing grain:", grain._id);                                                  // 110
                                                                                                      // 111
        // We take a highly synchronous approach here so that the logging is understandable.          // 112
        var supervisor = waitPromise(sandstormBackend.startGrain(                                     // 113
            grain.userId, grain._id, grain.packageId, dummyCommand, false, false)).supervisor;        // 114
        try {                                                                                         // 115
          waitPromise(supervisor.shutdown());                                                         // 116
          throw new Error("shutdown() didn't disconnect!");                                           // 117
        } catch (err) {                                                                               // 118
          if (err.kjType !== "disconnected") throw err;                                               // 119
        }                                                                                             // 120
      } catch (err) {                                                                                 // 121
        console.log(err.stack);                                                                       // 122
      }                                                                                               // 123
    });                                                                                               // 124
  }                                                                                                   // 125
}                                                                                                     // 126
                                                                                                      // 127
// This must come after all the functions named within are defined.                                   // 128
// Only append to this list!  Do not modify or remove list entries;                                   // 129
// doing so is likely change the meaning and semantics of user databases.                             // 130
var MIGRATIONS = [                                                                                    // 131
  updateLoginStyleToRedirect,                                                                         // 132
  enableLegacyOAuthProvidersIfNotInSettings,                                                          // 133
  denormalizeInviteInfo,                                                                              // 134
  mergeRoleAssignmentsIntoApiTokens,                                                                  // 135
  fixOasisStorageUsageStats,                                                                          // 136
];                                                                                                    // 137
                                                                                                      // 138
function migrateToLatest() {                                                                          // 139
  var applied = Migrations.findOne({_id: "migrations_applied"});                                      // 140
  var start;                                                                                          // 141
  if (!applied) {                                                                                     // 142
    // Migrations table is not yet seeded with a value.  This means it has                            // 143
    // applied 0 migrations.  Persist this.                                                           // 144
    Migrations.insert({_id: "migrations_applied", value: 0});                                         // 145
    start = 0;                                                                                        // 146
  } else {                                                                                            // 147
    start = applied.value;                                                                            // 148
  }                                                                                                   // 149
  console.log("Migrations applied: " + start + "/" + MIGRATIONS.length);                              // 150
                                                                                                      // 151
  for (var i = start ; i < MIGRATIONS.length ; i++) {                                                 // 152
    // Apply migration i, then record that migration i was successfully run.                          // 153
    console.log("Applying migration " + (i+1));                                                       // 154
    MIGRATIONS[i]();                                                                                  // 155
    Migrations.update({_id: "migrations_applied"}, {$set: {value: i+1}});                             // 156
    console.log("Applied migration " + (i+1));                                                        // 157
  }                                                                                                   // 158
}                                                                                                     // 159
                                                                                                      // 160
// Apply all migrations on startup.                                                                   // 161
Meteor.startup(migrateToLatest);                                                                      // 162
                                                                                                      // 163
////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-db'] = {
  SandstormDb: SandstormDb
};

})();

//# sourceMappingURL=sandstorm-db.js.map
