(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var _ = Package.underscore._;
var AppConfig = Package['application-configuration'].AppConfig;

/* Package-scope variables */
var SandstormEmail;

(function () {

/////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                             //
// packages/sandstorm-email/email.js                                                           //
//                                                                                             //
/////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                               //
var Future = Npm.require('fibers/future');                                                     // 1
var urlModule = Npm.require('url');                                                            // 2
var MailComposer = Npm.require('mailcomposer').MailComposer;                                   // 3
                                                                                               // 4
SandstormEmail = {};                                                                           // 5
                                                                                               // 6
var getSmtpUrl = function () {                                                                 // 7
  var setting = Settings.findOne({_id: "smtpUrl"});                                            // 8
  if (setting) {                                                                               // 9
    return setting.value;                                                                      // 10
  } else {                                                                                     // 11
    return process.env.MAIL_URL;                                                               // 12
  }                                                                                            // 13
};                                                                                             // 14
                                                                                               // 15
var makePool = function (mailUrlString) {                                                      // 16
  var mailUrl = urlModule.parse(mailUrlString);                                                // 17
  if (mailUrl.protocol !== 'smtp:')                                                            // 18
    throw new Error("Email protocol in $MAIL_URL (" +                                          // 19
                    mailUrlString + ") must be 'smtp'");                                       // 20
                                                                                               // 21
  var port = +(mailUrl.port);                                                                  // 22
  var auth = false;                                                                            // 23
  if (mailUrl.auth) {                                                                          // 24
    var parts = mailUrl.auth.split(':', 2);                                                    // 25
    auth = {user: parts[0] && decodeURIComponent(parts[0]),                                    // 26
            pass: parts[1] && decodeURIComponent(parts[1])};                                   // 27
  }                                                                                            // 28
                                                                                               // 29
  var simplesmtp = Npm.require('simplesmtp');                                                  // 30
  var pool = simplesmtp.createClientPool(                                                      // 31
    port,  // Defaults to 25                                                                   // 32
    mailUrl.hostname,  // Defaults to "localhost"                                              // 33
    { secureConnection: (port === 465),                                                        // 34
      // XXX allow maxConnections to be configured?                                            // 35
      auth: auth });                                                                           // 36
                                                                                               // 37
  pool._future_wrapped_sendMail = _.bind(Future.wrap(pool.sendMail), pool);                    // 38
  return pool;                                                                                 // 39
};                                                                                             // 40
                                                                                               // 41
// We construct smtpPool at the first call to Email.send, so that                              // 42
// Meteor.startup code can set $MAIL_URL.                                                      // 43
var pool;                                                                                      // 44
var configured = false;                                                                        // 45
                                                                                               // 46
Meteor.startup(function() {                                                                    // 47
  Settings.find({_id: "smtpUrl"}).observeChanges({                                             // 48
    removed : function () {                                                                    // 49
      configured = false;                                                                      // 50
    },                                                                                         // 51
    changed : function () {                                                                    // 52
      configured = false;                                                                      // 53
    },                                                                                         // 54
    added : function () {                                                                      // 55
      configured = false;                                                                      // 56
    }                                                                                          // 57
  });                                                                                          // 58
                                                                                               // 59
  // Accounts.emailToken is set to use "Email" by default. Change it to use our mail service.  // 60
  Accounts.emailToken.setEmailPackage("SandstormEmail");                                       // 61
});                                                                                            // 62
                                                                                               // 63
var getPool = function (smtpUrl) {                                                             // 64
  if (smtpUrl) {                                                                               // 65
    return makePool(smtpUrl);                                                                  // 66
  } else if (!configured) {                                                                    // 67
    configured = true;                                                                         // 68
    var url = getSmtpUrl();                                                                    // 69
    if (url) {                                                                                 // 70
      pool = makePool(url);                                                                    // 71
    }                                                                                          // 72
  }                                                                                            // 73
                                                                                               // 74
  return pool;                                                                                 // 75
};                                                                                             // 76
                                                                                               // 77
var next_devmode_mail_id = 0;                                                                  // 78
var output_stream = process.stdout;                                                            // 79
                                                                                               // 80
var devModeSend = function (mc) {                                                              // 81
  var devmode_mail_id = next_devmode_mail_id++;                                                // 82
                                                                                               // 83
  var stream = output_stream;                                                                  // 84
                                                                                               // 85
  // This approach does not prevent other writers to stdout from interleaving.                 // 86
  stream.write("====== BEGIN MAIL #" + devmode_mail_id + " ======\n");                         // 87
  stream.write("(Mail not sent; to enable sending, set the MAIL_URL " +                        // 88
               "environment variable.)\n");                                                    // 89
  mc.streamMessage();                                                                          // 90
  mc.pipe(stream, {end: false});                                                               // 91
  var future = new Future;                                                                     // 92
  mc.on('end', function () {                                                                   // 93
    stream.write("====== END MAIL #" + devmode_mail_id + " ======\n");                         // 94
    future['return']();                                                                        // 95
  });                                                                                          // 96
  future.wait();                                                                               // 97
};                                                                                             // 98
                                                                                               // 99
var smtpSend = function (pool, mc) {                                                           // 100
  pool._future_wrapped_sendMail(mc).wait();                                                    // 101
};                                                                                             // 102
                                                                                               // 103
// Old comment below                                                                           // 104
/**                                                                                            // 105
 * Send an email.                                                                              // 106
 *                                                                                             // 107
 * Connects to the mail server configured via the MAIL_URL environment                         // 108
 * variable. If unset, prints formatted message to stdout. The "from" option                   // 109
 * is required, and at least one of "to", "cc", and "bcc" must be provided;                    // 110
 * all other options are optional.                                                             // 111
 *                                                                                             // 112
 * @param options                                                                              // 113
 * @param options.from {String} RFC5322 "From:" address                                        // 114
 * @param options.to {String|String[]} RFC5322 "To:" address[es]                               // 115
 * @param options.cc {String|String[]} RFC5322 "Cc:" address[es]                               // 116
 * @param options.bcc {String|String[]} RFC5322 "Bcc:" address[es]                             // 117
 * @param options.replyTo {String|String[]} RFC5322 "Reply-To:" address[es]                    // 118
 * @param options.subject {String} RFC5322 "Subject:" line                                     // 119
 * @param options.text {String} RFC5322 mail body (plain text)                                 // 120
 * @param options.html {String} RFC5322 mail body (HTML)                                       // 121
 * @param options.headers {Object} custom RFC5322 headers (dictionary)                         // 122
 */                                                                                            // 123
                                                                                               // 124
// New API doc comment below                                                                   // 125
/**                                                                                            // 126
 * @summary Send an email. Throws an `Error` on failure to contact mail server                 // 127
 * or if mail server returns an error. All fields should match                                 // 128
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                                // 129
 * @locus Server                                                                               // 130
 * @param {Object} options                                                                     // 131
 * @param {String} options.from "From:" address (required)                                     // 132
 * @param {String|String[]} options.to,cc,bcc,replyTo                                          // 133
 *   "To:", "Cc:", "Bcc:", and "Reply-To:" addresses                                           // 134
 * @param {String} [options.subject]  "Subject:" line                                          // 135
 * @param {String} [options.text|html] Mail body (in plain text or HTML)                       // 136
 * @param {Object} [options.headers] Dictionary of custom headers                              // 137
 * @param {String} [options.smtpUrl] SMTP server to use. Otherwise defaults to configured one. // 138
 */                                                                                            // 139
SandstormEmail.send = function (options) {                                                     // 140
  var mc = new MailComposer();                                                                 // 141
                                                                                               // 142
  // setup message data                                                                        // 143
  // XXX support attachments (once we have a client/server-compatible binary                   // 144
  //     Buffer class)                                                                         // 145
  mc.setMessageOption({                                                                        // 146
    from: options.from,                                                                        // 147
    to: options.to,                                                                            // 148
    cc: options.cc,                                                                            // 149
    bcc: options.bcc,                                                                          // 150
    replyTo: options.replyTo,                                                                  // 151
    subject: options.subject,                                                                  // 152
    text: options.text,                                                                        // 153
    html: options.html                                                                         // 154
  });                                                                                          // 155
                                                                                               // 156
  _.each(options.headers, function (value, name) {                                             // 157
    mc.addHeader(name, value);                                                                 // 158
  });                                                                                          // 159
                                                                                               // 160
  SandstormEmail.rawSend(mc, options.smtpUrl);                                                 // 161
};                                                                                             // 162
                                                                                               // 163
/**                                                                                            // 164
 * @summary Sends a raw email with a MailComposer object.                                      // 165
 * Throws an `Error` on failure to contact mail server                                         // 166
 * or if mail server returns an error. All fields should match                                 // 167
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.                                // 168
 * @locus Server                                                                               // 169
 * @param {Object} mc A MailCompser object that you wish to send                               // 170
 * @param {String} smtpUrl SMTP server to use. If falsey, defaults to configured one.          // 171
*/                                                                                             // 172
SandstormEmail.rawSend = function (mc, smtpUrl) {                                              // 173
  var pool = getPool(smtpUrl);                                                                 // 174
  if (pool) {                                                                                  // 175
    smtpSend(pool, mc);                                                                        // 176
  } else {                                                                                     // 177
    throw new Error("SMTP pool is misconfigured.");                                            // 178
  }                                                                                            // 179
};                                                                                             // 180
                                                                                               // 181
/////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-email'] = {
  SandstormEmail: SandstormEmail
};

})();

//# sourceMappingURL=sandstorm-email.js.map
